//--------------------------------------------------------------------------------------
//	$Id: decay-filter-test.cc,v 1.1 2024/03/13 20:21:10 wolf Exp $
//--------------------------------------------------------------------------------------

#include <stdio.h>
#include <stdint.h>
#include <math.h>

#include "decay-compute-ks.h"
#include "decay-filter.h"

//--------------------------------------------------------------------------------------
//	config frequencies
//--------------------------------------------------------------------------------------

constexpr uint32_t Frequencies[] = {

	500, 1'000, 1'200, 1'500, 2'000, 3'000
};

static constexpr unsigned N_Frequencies =
				sizeof(Frequencies) / sizeof(*Frequencies);

//--------------------------------------------------------------------------------------
//	config half lifes (us): 1 ms ... 100 ms
//--------------------------------------------------------------------------------------

constexpr uint32_t HalfLifes[] = {

	1'000, 2'000, 5'000, 10'000, 20'000, 50'000, 100'000
};

static constexpr unsigned N_HalfLifes =
				sizeof(HalfLifes) / sizeof(*HalfLifes);

//--------------------------------------------------------------------------------------
//	main
//--------------------------------------------------------------------------------------

int
main(){
	for( unsigned uth = 0; uth < N_HalfLifes; ++uth ){

		const uint32_t th_us   = HalfLifes[ uth ];

		printf(	"+---------------+---------------------+-------------+---"
			"-------------+----------------+----------------+-------+\n" );

		for( unsigned uf = 0; uf < N_Frequencies; ++uf ){

			const uint32_t f_hz = Frequencies[ uf ];

			const uint32_t n_steps = f_hz * th_us / 1'000'000u;

			const  int32_t ks      = compute_ks( f_hz, th_us );

			//--------------------------------------------------------------
			//	simulation: erster Schritt
			//--------------------------------------------------------------

			int32_t out = 0;
			int32_t in  = INT32_MAX;

        		decay_filter( in, ks, &out );

			const int32_t first_out = out;

			//--------------------------------------------------------------
			//	simulation: übrige Schritte
			//--------------------------------------------------------------

			in = 0;

			for( unsigned step = 2; step <= n_steps; ++step ){

        			decay_filter( in, ks, &out );
			}

			const int32_t last_out = out;

			const float percent = 100.0 * last_out / first_out;

			//--------------------------------------------------------------
			//	simulation: übrige Schritte
			//--------------------------------------------------------------

			printf(
"| freq: %4u Hz | halflife: %6u us | steps: %4u | ks:%11u | 1st:%10u | lst:%10u |%6.2f |\n",
				f_hz, th_us, n_steps, ks, first_out, last_out, percent );
		}
	}

	printf(	"+---------------+---------------------+-------------+---"
		"-------------+----------------+----------------+-------+\n" );

	return 0;
}

//--------------------------------------------------------------------------------------
//	$Id: decay-filter-test.cc,v 1.1 2024/03/13 20:21:10 wolf Exp $
//--------------------------------------------------------------------------------------
